/* mutplot_input_sequence.h
 * scans sequence files in various formats, lists sequences
 * and displays other information.
 *  Original C version: 92/09/18   
 */

#include <stdio.h> 
#include <string.h>
#include <stdlib.h>             
#include <math.h>

/*     integer:  itype     file format code:
                                             1  -  GENBANK
                                             2  -  EMBL
                                             3  -  PIR
                                             4  -  STANFORD/IG
                                             5  -  GCG
                                             6     FASTA 

               type       ARRAY of sequence format types (as above)
               gcglen     ARRAY of (GCG) sequence lengths
               idno       counter for sequence identifiers found
               line       counter for records read from a file
               maxlen     maximum allowable sequence length
               pline      ARRAY of pointers to records that 
                          immediately precede sequence code
               opt        index of sequence chosen by user
               seqlen     actual length of the retrieved sequence
               seqone     index of first usable sequence

               startrec, startid, startlen, endid, start, stop:
                          markers used in extracting data from the input

  characters:  c          single character of sequence code
               filnam     string of 40 - user supplied input file name
               rec        string of 90 - a single record from a file
               seq        ARRAY of single characters - the sequence
               seqid      ARRAY of strings of 30 - all sequence id's

*/

typedef short Si;

struct sequence_holder {
  char name[31];
  char *sequence; /* bases of sequence */
  Si  *sequence_num;  /* numbers corrsp to bases */
};

#define FALSE 0
#define TRUE  1

/* for sequence_num
 * 1 for A
 * 2 for C
 * 3 for G
 * 4 for U/T
 * 8 0 for other, counted as insertion
 */

/* translator from small number to big numbers=alignment position
 * rev_translator from big numbers=alignment position to small_numbers
 * small number i for  the ith base of the original sequence 
 */

#define BIG_CONVERT_FACTOR 1000

/* define symbolic constants */

#define MAXSEQS 2000
#define MAXLEN 800

FILE *inf;              /* global variables */

/* holds sequence data, 0 is not used value 1 is for sequence 1, */
struct sequence_holder *g_data;

int g_sequence_length = 1;
int g_number_sequences = 1;
int g_displayed_sequence = 1;
int *g_displayed_sequence_numbering; /* small to big */
int *g_reverse_displayed_sequence_numbering; /* big to small */

/* Use 1 for A, 2 for C, 3 for G, 4 for U/T and 0 for insertions */ 
float *g_fI[5];

Si *g_big_matrix[MAXLEN];

void open_infil (char *filename) {
  inf = fopen(filename, "r");  
  if(inf==NULL) {
    printf("Error!\tThe file %s could not be opened\n",filename);
    try_exit(32);
  } 
}

void getid (char *rec, char *id, Si sid, char *srec, char *string) {
  char *no_id = "unidentified sequence";
  /* extract sequence id from current record
   * copy string from start position of id
   * locate end of id
   * If expected terminator not found, end id at new line char.
   * Calculate length of id string
   * Check length
   * If no id, label as unidentified or strip blanks from id of
   * useable length or truncate long id to useable length.
   */
  Si  end;
  strncpy(id, &rec[sid], 29); 
  end = strstr(&rec[sid], string) - srec;      
  if (end < 0)                
    end = strstr(&rec[sid], "\n") - srec;
  end -= sid;                     
  if (end == 0)                            
    strcpy(id, no_id);            
  else if (end > 0 && end < 30)           
    id[end] = 0;  
  else 
    id[29] = 0;
}

Si scan_seq ( char seqid[MAXSEQS][30], Si type[MAXSEQS], Si gcglen[MAXSEQS], 
	      Si pline[MAXSEQS]) {
  /* scan input file and identify data format */
  char rec[90]; 
  char *startrec;
  Si line, itype, startid, startlen, idno;
  line = idno = itype = 0;
  while (fgets(rec, 90, inf) != NULL && idno+1 < MAXSEQS) {
    ++line;
    startrec = strstr(rec,rec);     /* store array address as an */
    if (strstr(rec, "..") != NULL && strstr(rec, "Length:") != NULL) {
      if (idno == 0) {
	idno = 1;
	printf("GCG format found\n");
      }   
      if (pline[idno] > 0 && (line - pline[idno]) > 3) {
	++idno;
	printf("Alternative format converted to GCG\n");
      } 
      type[idno] = itype = 5;     /* store data for sequence retrieval */
      pline[idno] = line + 1;
      startid = strspn(rec, " ");
      getid (rec, seqid[idno], startid, startrec, "Length:");
      startlen = strstr(&rec[startid], "Length:") - startrec;
      sscanf (&rec[startlen+7], "%hd", &gcglen[idno]);
    } else if (strstr(rec, "LOCUS") == startrec) {
      if(idno==1) printf("GENBANK,LOCUS found\n"); 
      ++idno;
      type[idno] = itype = 1;
      getid (rec, seqid[idno], 12, startrec, " ");
      if(idno==1) printf("GENBANK name is %s\n",seqid[idno]); 
    } else if (strstr(rec, "ORIGIN") == startrec && itype == 1) { 
      if(idno==1) printf("GENBANK, ORIGIN found\n"); 
      if (pline[idno])   /* first or subsequent GENB sequence? */
	++idno;
      type[idno] = 1;
      pline[idno] = line;                        
      } else if (strstr(rec, "ID   ") == startrec) {
	if(idno==1) printf("EMBL format was found\n");  
	++idno;
	type[idno] = itype = 2;
	getid (rec, seqid[idno], 5, startrec, " ");
      } else if (strstr(rec, "SQ   ") == startrec) {
	if (pline[idno])   /* first or subsequent EMBL sequence? */
	  ++idno;
	type[idno] = 2;
	pline[idno] = line;                        
      } else if (rec[0]=='>' && rec[1]=='D' && rec[2]=='L' && rec[3]==';') { 
	++idno;
	type[idno] = itype = 3;
	if(idno==1) printf("PIR format?\n");
	pline[idno] = line + 1;
	startid = strchr(rec, ';') + 1 - startrec;
	getid (rec, seqid[idno], startid, startrec, "\n");
    } else if (rec[0] == '>')  {
      ++idno;
      type[idno] = itype = 6;
      if(idno==1) printf("FASTA format?\n");
      pline[idno] = line;
      startid = strchr(rec, '>') + 1 - startrec;
      getid (rec, seqid[idno], startid, startrec, "\n");
    } else if (rec[0] == ';') {
      itype = -4;
      if (idno==1) printf("STANFORD/IG format was found\n"); 
    } else if (itype == -4) {
      ++idno;
      type[idno] = itype = 4;
      pline[idno] = line;
      getid (rec, seqid[idno], 0, startrec, "\n");
    }
  }
  return idno;
}

Si formid (char *id, char *seq, Si maxlen,char *filename,
	   int *number_of_sequences) {
  char rec[90], c;
  char seqid[MAXSEQS][30];
  Si type[MAXSEQS], gcglen[MAXSEQS], pline[MAXSEQS];
  Si  i, start, stop, seqlen, opt,idno;
  int master_length = -1;
  open_infil (filename); /* open input file */
  for (i = 0; i < MAXSEQS; i++) 
    type[i] = pline[i] = gcglen[i] = 0; 
  /* check for useable sequence(s) */
  *number_of_sequences = scan_seq(seqid,type,gcglen,pline);
  idno = *number_of_sequences;
  /* scan input file 
   * check if there are any valid sequences
   * idno is total number of sequences? */
  for (i = 0; i <= idno && pline[i] == 0; i++);
  if (i > idno) {
    printf("3. No recognizably formatted sequence found...\n");
    try_exit(33);
  }
  /* retrieve all sequences */
    g_data=(struct sequence_holder *)
      malloc(sizeof(struct sequence_holder)*(MAXSEQS+1));
    printf("Summary of %d sequences\n",idno);
    if(g_data==NULL) {
      printf("Insufficient memory from mutplot within formid, g_data\n");
      try_exit(34);
    }
    for(opt=1;opt<=idno;opt++) {
      rewind (inf);    /* rewind input file and set controls for retrieval */
      if((type[opt]==1)||(type[opt]==5))
	start=10;
      else
	start=0;
      if( (type[opt]==5)&& (gcglen[opt]<maxlen))
	stop=gcglen[opt];
      else
	stop=maxlen;
      i = 0; /* scan records preceding sequence code */
      while (++i <= pline[opt] && fgets(rec, 90, inf) != NULL);
    if (i < pline[opt]) { /* unexpected EOF */
      printf("2.  Unexpected end of file encountered...\n");
      printf("    No sequence data retrieved.\n");
      try_exit(35);  /* retrieval unsuccessful, skip to new loop iteration */
    }
    seqlen = 0;                             /* initialize length counter */
    while (seqlen < stop) {
      if ((fgets(rec, 90, inf) == NULL)) {
	if(type[opt]==6)
	  stop=seqlen;
	else {
	  printf("1.  Unexpected end of file encountered... type[opt] is %d\n"
		  ,type[opt]);
	  try_exit(36);
	}
      }
      else if ((rec[0] == '/')||(rec[0]=='>')) { /* check for terminator */
	stop = seqlen;
      }
      else {                      /* store sequence code from the record */
	for (i = 0; rec[i] != '\n' && seqlen < stop; i++) {
	  if (i < start)
	    continue;
	  c = rec[i];                       /* examine a character */
	  if (c == '1' || c == '2' || c == '*')      /* check for */
	    stop = seqlen;                         /* terminator */
	  else if ((c != ' ') &&  (!isdigit(c))) {
	    if((c=='-')||(c=='~'))
	      c='.';
	    if(islower(c))
	      c=toupper(c);
	    if(c=='T')
	      c='U';
	    seq[seqlen++] = c;     /* store code character */
	  }
	} 
      }
    }
    seq[seqlen] = 0;        /* add terminator to the character string */
    strcpy (id, seqid[opt]);   /* select id string for chosen sequence */
    printf("Sequence %d",opt);
    /* error after here and */ 
    if(master_length>0) {
      if(seqlen!=master_length) {
	printf("Error in sequence lengths!\n");
	printf("Sequence %s has length %d but previous has length %d\n",
	       id,seqlen,master_length);
	try_exit(37);
      }
    }
    else
      master_length = seqlen; 
    printf("\t%s\n",seqid[opt]);
    g_data[opt].sequence=(char *)malloc(sizeof(char)*(MAXLEN+2));
    g_data[opt].sequence_num = (Si *)malloc(sizeof(Si)*(MAXLEN+2));
    if( (g_data[opt].sequence==NULL) || (g_data[opt].sequence_num==NULL)) {
      printf("Insufficient memory from mutplot, formid data malloc\n");
      try_exit(38);
    } 
    strcpy(g_data[opt].name,seqid[opt]);
    strcpy(g_data[opt].sequence,"5");
    strcat(g_data[opt].sequence,seq);
    }
    g_displayed_sequence_numbering = (int *)malloc(sizeof(int)*(MAXLEN+2));
    g_reverse_displayed_sequence_numbering=
      (int *)malloc(sizeof(int)*(MAXLEN+2)); 
    if(g_displayed_sequence_numbering==NULL) {
      printf("Insufficient memory from mutplot,formid dis_seq_num malloc\n");
      try_exit(39);
    }
    if(g_reverse_displayed_sequence_numbering==NULL) {
      printf("Insufficient memory from mutplot,formid rev_dis_seq_num malloc\n");
      try_exit(40);
    }
    for(i=0;i<=4;i++) {
      g_fI[i]=(float *)malloc(sizeof(int)*(MAXLEN+2));
      if(g_fI[i]==NULL) {
	printf("Insufficient memory from mutplot,formid g_fI[%d]\n",i);
	try_exit(41);
      }
    }
    for(i=1;i<=MAXLEN;i++) {
      g_big_matrix[i]=(Si *)malloc(sizeof(Si)*(MAXLEN+2));
      if(g_big_matrix[i]==NULL) {
	printf("Insufficient memory from mutplot,formid g_big_matrix[%d]\n",i);
	try_exit(42);
      }
    }
    return master_length; /* return to main */
}

void read_all_sequences(char *file_name) {
  char id[30];
  int number_of_sequences;
  int length_of_sequences;
  char seq[10000];
  Si maxlen;
  maxlen=MAXLEN;
  length_of_sequences = formid(id, seq, maxlen, file_name, 
			       &number_of_sequences);
  g_sequence_length=length_of_sequences;
  g_number_sequences=number_of_sequences;
  printf("Return Values:\n");
  printf("Number of sequences:\t%d\n",number_of_sequences);
  printf("Alignment Length:\t%d\n",length_of_sequences);
}

void set_fi_data(void) {
  int i,sequence;
  char c;
  /* initialize fi to zero */
  for(i=1;i<=g_sequence_length;i++) {
    g_fI[0][i] = 0.;
    g_fI[1][i] = 0.;
    g_fI[2][i] = 0.;
    g_fI[3][i] = 0.;
    g_fI[4][i] = 0.;
  } 
  for(sequence=1;sequence<=g_number_sequences;sequence++) { 
    for(i=1;i<=g_sequence_length;i++) {
      c=g_data[sequence].sequence[i];
      if(c=='A') {
	g_fI[1][i] = g_fI[1][i] + 1.;          
	g_data[sequence].sequence_num[i]=(Si)1;
      } else if(c=='C') {
	g_fI[2][i] = g_fI[2][i] + 1.;
	g_data[sequence].sequence_num[i]=(Si)2;
      } else if(c=='G') {
	g_fI[3][i] = g_fI[3][i] + 1.;
	g_data[sequence].sequence_num[i]=(Si)3;
      } else if(c=='U') {
	g_fI[4][i] = g_fI[4][i] + 1.;
	g_data[sequence].sequence_num[i]=(Si)4;
      } else {
	g_data[sequence].sequence_num[i]=(Si)0;
	g_fI[0][i] = g_fI[0][i] + 1.;
      }
    }
  }
 for(i=1;i<=g_sequence_length;i++) {
   g_fI[0][i]=g_fI[0][i]/(float)g_number_sequences;
   g_fI[1][i]=g_fI[1][i]/(float)g_number_sequences;
   g_fI[2][i]=g_fI[2][i]/(float)g_number_sequences;
   g_fI[3][i]=g_fI[3][i]/(float)g_number_sequences;
   g_fI[4][i]=g_fI[4][i]/(float)g_number_sequences;
 }
}

void set_fij(float fij[5][5],int row,int column) { /* assign fij of formula */
  int i,j;
  int sequence;
  Si base_1,base_2;
  for (i=0;i<=4;i++) {
    for (j=0;j<=4;j++) {
      fij[i][j] = 0.;
    }
  }       
  for(sequence=1;sequence<=g_number_sequences;sequence++) {
    base_1=g_data[sequence].sequence_num[row];
    base_2=g_data[sequence].sequence_num[column];
    fij[base_1][base_2] = fij[base_1][base_2] + 1.;
  }
  for(i=0;i<=4;i++) {
    for(j=0;j<=4;j++) 
      fij[i][j]=fij[i][j]/g_number_sequences;
  }
}

void set_fij_int(int fij[5][5],int row,int column) { 
  int i,j;
  int sequence;
  Si base_1,base_2;
  for(i=0;i<=4;i++) {
    for(j=0;j<=4;j++) {
      fij[i][j]=0;
    }
  }       
  for(sequence=1;sequence<=g_number_sequences;sequence++) {
    base_1=g_data[sequence].sequence_num[row];
    base_2=g_data[sequence].sequence_num[column];
    fij[base_1][base_2]=fij[base_1][base_2]+1;
  }
}

void set_big_matrix(float low,float conserved_min) {
  int i_row, j_col;
  double log2; /* used to convert natural log to log base 2 */
  int N1,N2;
  float fij[5][5]; /* 0 for insertion, not used, 1 = A, 2 = C, 3 = G, 4 = U 
		    * e.g.: fij[1][3] is count of A.G base pairs
		    * in row i, column j */
  double temp_sum;
  int best_row = 1, best_col = 1, worst_row = 1, worst_col = 1;
  /* April 19, 2007. M. Zuker adjusts the computation so that rows where
   * both columns are gaps ( = 0) are not counted. This is an after the fact
   * correction for computing frequencies by dividing by the total number
   * of sequences in the alignment. The initial frequencies are left
   * uncorrected, since the output and/or display clearly shows what fractions
   * come from gap-gap rows.
   * fcij is the frequency of non gap-gap rows in the alignment. If it is zero,
   * then the computation below that uses 1/fcij is never done.
   */
  float max = -10000., min = 10000., fcij = 1.;
  log2 = log(2);
  printf("Setting Hij\n");
  for(i_row=1; i_row<=g_sequence_length; i_row++) {
    if((i_row%100)==0)
      printf("Current progress: Finished row %d\n",i_row);
    g_big_matrix[i_row][i_row] = (Si) 0;
    g_big_matrix[i_row][i_row+1] = (Si) 0;
    for(j_col=(i_row+2); j_col<=g_sequence_length; j_col++) {
      g_big_matrix[i_row][j_col] = (Si) 0;
      temp_sum = 0;
      /* assign fij of formula for this specific i_row,j_col*/
      set_fij(fij,i_row,j_col);
      fcij = 1.0 - fij[0][0]; /* M. Zuker correction. fcij = 1. in original */
      for(N1=1;N1<=4;N1++) {
	if (g_fI[N1][i_row]>0) {
	  for (N2=1;N2<=4;N2++) {
	    if( !(fij[N1][N2]==0.) && !(g_fI[N2][j_col]==0.))
	      temp_sum += (fij[N1][N2]/fcij)*
		log(fcij*fij[N1][N2]/(g_fI[N1][i_row]*g_fI[N2][j_col]));
	  }
	}
      }  
      temp_sum = temp_sum/log2;
      if(min>temp_sum) {
	min=temp_sum;
	worst_row = i_row;
	worst_col = j_col;
      }
      if(max<temp_sum) {
	max = temp_sum;
	best_row = i_row;
	best_col = j_col;
      }
      if (temp_sum<low) {
	if ((fij[2][3]>conserved_min)||(fij[3][2]>conserved_min))
	  temp_sum = 2.9;
	else if ((fij[1][4]>conserved_min)||(fij[4][1]>conserved_min))
	  temp_sum = 2.5;
	else if ((fij[3][4]>conserved_min)||(fij[4][3]>conserved_min))
	  temp_sum = 2.3;
      }
      g_big_matrix[i_row][j_col] = (Si)(BIG_CONVERT_FACTOR*temp_sum+.5);
    } 
  }
  printf("Maximum H(%d,%d) is %f.\tMinimum H(%d,%d) is %f\n",
	 best_row,best_col,max, worst_row,worst_col,min);
}

int g_ref_sequence_length; /* number of bases in reference sequence */

void set_numbering(int which_sequence) {
  int bases_used;
  int current_position;
  bases_used = 0;
  /* position j of the array g_displayed_sequence_numbering
   * gives the corresponding smaller position of the displayed sequence */ 
  for(current_position=1; current_position<=g_sequence_length;
      current_position++) {
    if(g_data[which_sequence].sequence[current_position]!='.') {
      bases_used++;
      g_displayed_sequence_numbering[current_position]=bases_used;
      g_reverse_displayed_sequence_numbering[bases_used]=
	current_position;
    } else
      g_displayed_sequence_numbering[current_position]=0;
  }
  g_ref_sequence_length=bases_used;
}

void find_sequence_by_name(char *name) {
  int found = FALSE, sequence_number = 1, test;
  char adjusted_name[120];
  /* test first 10 characters
   * put spaces onto both names to make them the same length
   * some formats leave spaces in the name, but not all
   */
  strcat(name,"          ");
  while((found==FALSE)&&(sequence_number<=g_number_sequences)) {
    strcpy(adjusted_name,g_data[sequence_number].name);
    strcat(adjusted_name,"          ");
    test=strncmp(name,adjusted_name,10);
    if(test==0)
      found=TRUE;
    else
      sequence_number++;
  }
  if(!found)
    printf("No name match was found\n");
  else {
    g_displayed_sequence=sequence_number; 
    set_numbering(g_displayed_sequence); 
  }
}

void initialize_sequence_data(char *file_name,float low, float min) {
  read_all_sequences(file_name); 
  set_fi_data();
  set_big_matrix(low,min);
  g_displayed_sequence=1;
  set_numbering(g_displayed_sequence);
  printf("When Hij < %.3f and a base pair occurs in more than %.2f percent\n",
	 low,100.*min);
  printf("of all sequences, it will be counted as conserved.\n");
  printf("For Conserved Base Pairs:\n");
  printf("G-C & C-G base pairs H(i,j) assigned 2.9\n");
  printf("A-U & U-A base pairs H(i,j) assigned 2.5\n");
  printf("G-U & U-G base pairs H(i,j) assigned 2.3\n");
}

FILE *id_fp;
int open_id_file(char *file_name) {    /* open specified output file */
  if ((id_fp = fopen(file_name, "w")) == NULL) {
    printf("Error!\tCould not open id_list file: %s\n", file_name);
    return(1);
  }
  return(0);
}

void general_id_list(char *id_filename,int row,int col, char *data_filename) {
  char *base_type=".ACGU";
  int type_row,type_col;
  int fij[5][5];
  float fij_float[5][5];
  int sequence;
  char type_row_char,type_col_char;
  printf("Listing of files by base content for (%d,%d) of %s in %s\n",
	 row,col,data_filename,id_filename);
  set_fij_int(fij,row,col);
  for(type_row=0;type_row<=4;type_row++) {
    for(type_col=0;type_col<=4;type_col++)
      fij_float[type_row][type_col]=(float)fij[type_row][type_col]/
	(float)g_number_sequences;
  }
  if(open_id_file(id_filename))
    return;
  /* Make table to show entries */
  fprintf(id_fp,"\n                Table for(%d,%d) of %s",
	  row,col,data_filename);
  fprintf(id_fp,"\n\n\n                            Column: %d \n",col);
  fprintf(id_fp,"\n                Fij |   .   |   A   |   C  |    G   |");
  fprintf(id_fp,"U   | Total |\n              __________________________");
  fprintf(id_fp,"_____________________________\n                .   | %.2f",
	  fij_float[0][0]);
  fprintf(id_fp,"| %.2f  | %.2f  | %.2f  | %.2f  | %.2f  |",
	  fij_float[0][1],fij_float[0][2],fij_float[0][3],fij_float[0][4],
	  fij_float[0][0]+fij_float[0][1]+fij_float[0][2]+fij_float[0][3]
	  +fij_float[0][4]);
  fprintf(id_fp,"\n              ___________________________________________");
  fprintf(id_fp,"____________\n                A   | %.2f  | %.2f  | %.2f  |",
	  fij_float[1][0],fij_float[1][1],fij_float[1][2]);
  fprintf(id_fp,"%.2f  | %.2f  | %.2f  |", fij_float[1][3],fij_float[1][4],
	  fij_float[1][0]+fij_float[1][1]+fij_float[1][2]+
	  fij_float[1][3]+fij_float[1][4]);
  fprintf(id_fp,"\n              ___________________________________________");
  fprintf(id_fp,"____________\n  Row: %4d     C   | %.2f  | %.2f  | %.2f  | ",
	  row,fij_float[2][0],fij_float[2][1],fij_float[2][2]);
  fprintf(id_fp,"%.2f  | %.2f  | %.2f  |",fij_float[2][3],fij_float[2][4],
	  fij_float[2][0]+fij_float[2][1]+fij_float[2][2]+
	  fij_float[2][3]+fij_float[2][4]);
  fprintf(id_fp,"\n              _______________________________________________________");
  fprintf(id_fp,
	  "\n                G   | %.2f  | %.2f  | %.2f  | %.2f  | %.2f  | %.2f  |",
	  fij_float[3][0],fij_float[3][1],fij_float[3][2],
	  fij_float[3][3],fij_float[3][4],
	  fij_float[3][0]+fij_float[3][1]+fij_float[3][2]+
	  fij_float[3][3]+fij_float[3][4]);
  fprintf(id_fp,
	  "\n              _______________________________________________________");
  fprintf(id_fp,
	  "\n                U   | %.2f  | %.2f  | %.2f  | %.2f  | %.2f  | %.2f  |", 
	  fij_float[4][0],fij_float[4][1],fij_float[4][2],
	  fij_float[4][3],fij_float[4][4],
	  fij_float[4][0]+fij_float[4][1]+fij_float[4][2]+
	  fij_float[4][3]+fij_float[4][4]);
  fprintf(id_fp,
	  "\n              _______________________________________________________");
  fprintf(id_fp,
	  "\n              Total | %.2f  | %.2f  | %.2f  | %.2f  | %.2f  |       |",
	  fij_float[0][0]+fij_float[1][0]+fij_float[2][0]+fij_float[3][0]+fij_float[4][0],
	  fij_float[0][1]+fij_float[1][1]+fij_float[2][1]+fij_float[3][1]+fij_float[4][1],
	  fij_float[0][2]+fij_float[1][2]+fij_float[2][2]+fij_float[3][2]+fij_float[4][2],
	  fij_float[0][3]+fij_float[1][3]+fij_float[2][3]+fij_float[3][3]+fij_float[4][3],
	  fij_float[0][4]+fij_float[1][4]+fij_float[2][4]+fij_float[3][4]+fij_float[4][4]);
  fprintf(id_fp,
	  "\n              _______________________________________________________");
  
  /* List of sequences for each type of pair */
  fprintf(id_fp,
	  "\n\n\n\n______________________________________________________________________________");
  fprintf(id_fp,"\n\n        id list for (%d,%d) of %s\n\n",row,col,data_filename);
  for(type_row=0;type_row<=4;type_row++) {
    for(type_col=0;type_col<=4;type_col++) {
      type_row_char=base_type[type_row];
      type_col_char=base_type[type_col];
      fprintf(id_fp,
	      "\n%c-%c has %d entries [%.1f%%]_______________________________\n",
	      type_row_char,type_col_char,fij[type_row][type_col],
	      fij_float[type_row][type_col]*100.);
      if(fij[type_row][type_col]!=0) {
	for(sequence=1;sequence<=g_number_sequences;sequence++) {
	  if((g_data[sequence].sequence_num[row]==type_row)&&
	     (g_data[sequence].sequence_num[col]==type_col)) {
	    fprintf(id_fp,"%s\n",g_data[sequence].name);
	  }
	}
      }
    }
  }
  fclose(id_fp);
}
